/*******************************************************************************
|
| Macro Name:      boxplotindividual
|
| Macro Version:   1.1
|
| SAS Version:     9.1.3
|
| Created By:      Andy Miskell
|
| Date:            July 5, 2011
|
| Macro Purpose:   
|
| Macro Design:    Procedure Style
|
| Input Parameters:
|
| NAME               DESCRIPTION                            REQ/OPT  DEFAULT
| -----------------  -------------------------------------  -------  ----------
|
|
|******************************************************************************
*******************************************************************************/

%macro pd_boxplotindividual(libname=,   /* Not Required: area where dataset is located.  If a work dataset, 
                                           then leave blank */
                    dsn=lab,            /* Required: dataset name with no library preceding it */
					byvar=,             /* Not Required: one or multiple variables to use as by-variables */
			        xvar=,              /* Required: one variable or two variables - if two variables, 
					                       second variable must be decode of first variable */
			        yvar=,              /* Required: one variable */
					zvar=,              /* Required: Treatment variable or other variable to break graph into 
					                       multiple compartments.  One variable or two variables - if two variables, 
					                       second variable must be decode of first variable */
                    formats=,           /* Not Required: 3 formats maximum.  If two variables have the same format, 
					                       they must be assigned uniquely (i.e. "day best. conc best." and NOT 
					                       "day conc best.") */
					title=,             /* Not Required: Title of Graph */
      			    xaxislabel=,        /* Not Required: X-axis label */
			        yaxislabel=,        /* Not Required: Y-axis label.  If parameter is blank and by variables 
					                       are specified, then by variables will be y-axis label */
					hrefline=,          /* Not Required: Horizontal reference line desired (i.e. line at 0 if 
					                       change from baseline).  May have more than one number separated by spaces */
					outputpathfilename= /* Required: Path and file name of output file (must be a PDF extension */
                    );

%annomac;
options spool;

%let title = %nrbquote(&title.);
%let xaxislabel = %nrbquote(&xaxislabel.);
%let yaxislabel = %nrbquote(&yaxislabel.);
%let outputpathfilename = %nrbquote(&outputpathfilename.);

/* Read formats into a dataset and add to x,y,z,by variable lists and macros */

/* Read in data */
%if &libname^= %then %do;
  libname topbot "&libname" access=readonly;

  data work.pdlab;
    set topbot.&dsn;
  run;
%end;
%else %do;
  data work.pdlab;
    set work.&dsn;
  run;
%end;
/* Parse out macro parameters for x-axis, treatment groupings, and by-variables */
%local xvarcount zvarcount byvarcount firstby lastby firstx lastx firstz lastz;

%if %scan(&xvar,2,' ')^= %then %do;
  %let xvarcount=2;
%end;
%else %do;
  %let xvarcount=1;
%end;
%if %scan(&zvar,2,' ')^= %then %do;
  %let zvarcount=2;
%end;
%else %do;
  %let zvarcount=1;
%end;
%if %scan(&byvar,2,' ')^= %then %do;
  %let byvarcount=2;
%end;
%else %if %scan(&byvar,1,' ')^= %then %do;
  %let byvarcount=1;
%end;
%else %do;
  %let byvarcount=0;
%end;

%if &xvarcount=1 %then %do;
  %let firstx=&xvar;
  %let lastx=&xvar;
%end;
%if &zvarcount=1 %then %do;
  %let firstz=&zvar;
  %let lastz=&zvar;
%end;
%if &byvarcount=1 %then %do;
  %let firstby=&byvar;
  %let lastby=&byvar;
%end;
%if &xvarcount=2 %then %do;
  %let firstx=%scan(&xvar,1,' ');
  %let lastx=%scan(&xvar,2,' ');
%end;
%if &zvarcount=2 %then %do;
  %let firstz=%scan(&zvar,1,' ');
  %let lastz=%scan(&zvar,2,' ');
%end;
%if &byvarcount=2 %then %do;
  %let firstby=%scan(&byvar,1,' ');
  %let lastby=%scan(&byvar,2,' ');
%end;

/* Enter format parameter values into dataset and other macro parameters */
%if &formats^= %then %do;
  %local forvar1 format1;
  /* Scan formats parameter.  Find first format */
  %if %scan(&formats,2,' ')^= %then %do;
    %let forvar1=;
    %let format1=;
	/* Determine variable name and format name of first format */
    %let forvar1=%scan(&formats,1,' ');
    %let format1=%scan(&formats,2,' ');

	/* Create decoded variable of first format named formvariable1 */
	data pdlab;
	  set pdlab;
	  length formvariable1 $256;
	  formvariable1=put(&forvar1,?? &format1.);
	run;

	/* Determine if first format is meant for by-value, x-axis, or treatment grouping */
	/* For whichever one it is meant for, replace the effected macro parameters with new
	   decoded variable */
    %if %index(&byvar,&forvar1)^=0 and &firstby=&forvar1 %then %do;
	  %let byvarcount=2;
	  %let lastby=formvariable1;
	  %let byvar=&firstby formvariable1;
	%end;
    %if %index(&xvar,&forvar1)^=0 and &firstx=&forvar1 %then %do;
	  %let xvarcount=2;
	  %let lastx=formvariable1;
	  %let xvar=&firstx formvariable1;
	%end;
    %if %index(&zvar,&forvar1)^=0 and &firstz=&forvar1 %then %do;
	  %let zvarcount=2;
	  %let lastz=formvariable1;
	  %let zvar=&firstz formvariable1;
	%end;

  %end;

  /* Scan formats parameter.  Find second format */
  %if %scan(&formats,4,' ')^= %then %do;
    %let forvar1=;
    %let format1=;
    %let forvar1=%scan(&formats,3,' ');
    %let format1=%scan(&formats,4,' ');

	/* Create decoded variable of second format named formvariable2 */
	data pdlab;
	  set pdlab;
	  length formvariable2 $256;
	  formvariable2=put(&forvar1,?? &format1.);
	run;

	/* Determine if second format is meant for by-value, x-axis, or treatment grouping */
	/* For whichever one it is meant for, replace the effected macro parameters with new
	   decoded variable */
    %if %index(&byvar,&forvar1)^=0 and &firstby=&forvar1 %then %do;
	  %let byvarcount=2;
	  %let lastby=formvariable2;
	  %let byvar=&firstby formvariable2;
	%end;
    %if %index(&xvar,&forvar1)^=0 and &firstx=&forvar1 %then %do;
	  %let xvarcount=2;
	  %let lastx=formvariable2;
	  %let xvar=&firstx formvariable2;
	%end;
    %if %index(&zvar,&forvar1)^=0 and &firstz=&forvar1 %then %do;
	  %let zvarcount=2;
	  %let lastz=formvariable2;
	  %let zvar=&firstz formvariable2;
	%end;

  %end;

  /* Scan formats parameter.  Find third format */
  %if %scan(&formats,6,' ')^= %then %do;
    %let forvar1=;
    %let format1=;
    %let forvar1=%scan(&formats,5,' ');
    %let format1=%scan(&formats,6,' ');

	/* Create decoded variable of third format named formvariable3 */
	data pdlab;
	  set pdlab;
	  length formvariable3 $256;
	  formvariable3=put(&forvar1,?? &format1.);
	run;

	/* Determine if third format is meant for by-value, x-axis, or treatment grouping */
	/* For whichever one it is meant for, replace the effected macro parameters with new
	   decoded variable */
    %if %index(&byvar,&forvar1)^=0 and &firstby=&forvar1 %then %do;
	  %let byvarcount=2;
	  %let lastby=formvariable3;
	  %let byvar=&firstby formvariable3;
	%end;
    %if %index(&xvar,&forvar1)^=0 and &firstx=&forvar1 %then %do;
	  %let xvarcount=2;
	  %let lastx=formvariable3;
	  %let xvar=&firstx formvariable3;
	%end;
    %if %index(&zvar,&forvar1)^=0 and &firstz=&forvar1 %then %do;
	  %let zvarcount=2;
	  %let lastz=formvariable3;
	  %let zvar=&firstz formvariable3;
	%end;

  %end;

%end;
/* End Formats part */

/* Find cross product of all x-axis values and treatment values to determine 
   length of x-axis */
proc sort data=pdlab out=tempxlab (keep=&xvar) nodupkey;
  by &xvar;
run;

proc univariate data=tempxlab noprint;
  var &firstx;
  output out=cntx n=xcnt;
run;

%local xcntr;

/* Macro variable to count number of timepoints for each treatment group */
data cntx;
  set cntx;
  call symput('xcntr', put(xcnt,?? 3.));
run;

proc sort data=pdlab out=tempzlab (keep=&zvar) nodupkey;
  by &zvar;
run;
  /* Get full cartesan product of timepoints and treatments */
  proc sql;
    create table fullproduct as    
    select tempxlab.*           
          ,tempzlab.*       
                           
    from tempxlab               
        ,tempzlab;              
  quit; 

proc sort data=fullproduct;
  by &zvar &xvar;
run;

/* Create x-axis indexing counting sequentially through the timepoints for each treatment */
/* At the end of each treatment, add .5 and add to the reference bar variable to later
   put reference lines on the graph */
data fullproduct;
  set fullproduct end=eof;
  by &zvar &xvar;
  length refbar $ 120;
  retain indexvar refbar;
  if _n_=1 then do;
    indexvar=1;
	refbar='.5';
  end;
  else do;
    if not first.&firstz then indexvar+1;
	else do;
	  indexvar+.5;
	  refbar=trim(left(refbar))||' '||left(put(indexvar,?? 5.1));
	  indexvar+.5;
	end;
  end;
  if eof then do;
    indexvar+.5;
    refbar=trim(left(refbar))||' '||left(put(indexvar,?? 5.1));
	refbar='('||trim(left(refbar))||')';
	indexvar=indexvar-.5;
  end;
run;

/* Read indexing created in previous data step into a format where
   the codes are the indexes and the decodes are the x-axis variable values.
   This format will be called xvariable. */
data fmtnam;
  set fullproduct;
  length label $256;
  call symput('refbars',trim(left(refbar)));
  start=indexvar;
  fmtname='xvariable';
  type='N';
  label=&lastx;
  label=left(label);
  codetype='TEXT';
  output;
  if _n_=1 then do;
    hlo='o';
	label=' ';
	output;
  end;
run;

/* Read xvariable. format into actual SAS format */
proc format cntlin=fmtnam;
run;

/* Merge indexes created back into main dataset */
proc sort data=pdlab out=pdlab;
  by &xvar &zvar;
run;

proc sort data=fullproduct out=fullproduct1 (keep=&xvar &zvar indexvar);
  by &xvar &zvar;
run;

data pdlab ;
  merge fullproduct1 pdlab (in=a);
  by &xvar &zvar;
  if a;
run; 

/* Find maximum index value so horizontal axis statement can have a maximum (&xmax macro variable) */
proc univariate data=fullproduct noprint;
  var indexvar;
  output out=maxdset max=maxvalue;
run;

data maxdset;
  set maxdset;
  maxvalue+.5;
  call symput('xmax',put(maxvalue,?? 5.1));
run;

/* Read treatment variables into a format zvariable. */
%if &zvar^= %then %do;
  proc sort data=pdlab out=zvartest (keep=&zvar) nodupkey;
    by &zvar;
  run;

  /* Get count of number of treatments */
  data zvartest;
    set zvartest;
	retain indexvar2;
	if _n_=1 then indexvar2=0;
	indexvar2+1;
  run;

  /* Create dataset for creating a format.  Codes are index of count of treatments.
     Decodes are the treatment names. */
  data fmtnam2;
    set zvartest;
    length label $256;
    start=indexvar2;
    fmtname='zvariable';
    type='N';
    label=&lastz;
    label=left(label);
    codetype='TEXT';
    output;
    if _n_=1 then do;
      hlo='o';
	  label=' ';
	  output;
    end;
  run;
 
  /* Read into zvariable. format */
  proc format cntlin=fmtnam2;
  run;

  /* Merge treatment indexing back into main dataset */
  proc sort data=pdlab out=pdlab;
    by &zvar;
  run;

  data pdlab;
    merge zvartest pdlab;
	by &zvar;
  run;

%end;

/* If by-variables are in macro call. */
%if &byvar^= %then %do;
  proc sort data=pdlab out=pdlabtest (keep=&byvar) nodupkey;
    by &byvar;
  run;

  /* Get count of number of by-groupings in data and index them. */
  data pdlabtest;
    set pdlabtest;
	retain indexvar1;
	if _n_=1 then indexvar1=0;
	indexvar1+1;
  run;

  /* Create dataset for creating a format.  Codes are index of count of by-groupings.
     Decodes are the by-groupings. */
  data fmtnam1;
    set pdlabtest;
    length label $256;
    start=indexvar1;
    fmtname='byvariable';
    type='N';
    label=&lastby;
    label=left(label);
    codetype='TEXT';
    output;
    if _n_=1 then do;
      hlo='o';
	  label=' ';
	  output;
    end;
  run;

  /* Read into byvariable. format */
  proc format cntlin=fmtnam1;
  run;

  /* Merge by-grouping indexing back into main dataset */
  proc sort data=pdlab out=pdlab;
    by &byvar;
  run;

  data pdlab;
    merge pdlabtest pdlab;
	by &byvar;
  run;

  /* Create macro variable &bycnt for number of by-groupings */
  proc univariate data=pdlabtest noprint;
    var indexvar1;
	output out=cntbyvar max=cntr;
  run;

  data cntbyvar;
    set cntbyvar;
	call symput('bycnt',put(cntr,?? 4.));
  run;

%end;

proc sort data=pdlab out=pdlab;
  by &byvar &xvar;
run;

/* Set up graphing options */
goptions reset=all;
goptions goutmode=replace; 
options orientation=landscape ls=108 ps=65;
title ' ';
/* Create header and title information */
proc gslide gout=grph_cat name="tbox";
  title1 j=c h=1.05 font=Courier " ";
run;
quit;

goptions goutmode=append;

proc greplay tc=grph_cat nofs ;
  tdef newtemp
  1/llx=0    lly=5   ulx=0   uly=93
    urx=100  ury=93  lrx=100  lry=5

  2/ llx=0   lly=0   ulx=0   uly=100 
     urx=100 ury=100 lrx=100 lry=0;
run;

/* Set up x and y axes as well as symbol statements */
axis1
	  label=(h=0.5 cm " ")
      order=(.5 to &xmax by .5)
      minor=none
	  major=none
      value=(rotate=0 angle=0 h=.55 cm f=Courier)
       ;
symbol;
symbol1 i=box00 c=blue;
symbol2 i=hilo c=blue;

axis2  
      label=(h=0.35 cm angle=90 rotate=0 " ") 
      offset=(0.25 in,0.25 in)
      minor=none
      value=(h=0.75 cm f=Courier)
       ;
/* If by-variables in the macro call. */
%if &byvar^= %then %do;
  /* One iteration per by-grouping.  Use the &bycnt variable created earlier to know how many 
     iterations are needed.  Use index variable for by-groupings created earlier (indexvar1)
     to subset data for only that by-group. */
  %local i;
  %do i=1 %to &bycnt;
    data anno;
	  set pdlab;
      if indexvar1=&i;
	  /* If no Y-axis label specified then use by-grouping as Y-axis label. */
	  call symput('ylab',trim(left(put(indexvar1,?? byvariable.))));
	run;

   %if &yaxislabel= %then %do;
     axis2  
      label=(h=0.5 cm angle=90 rotate=0 "&ylab") 
      offset=(0.25 in,0.25 in)
      minor=none
      value=(h=0.65 cm)
       ;
   %end;

	proc sort data=anno;
	  by indexvar2;
	run;

	/* Set up annotation */
	data anno;
	  set anno;
	  by indexvar2;
	  format indexvar1 byvariable. indexvar xvariable. indexvar2 zvariable. &formats;
	  length text $ 256 style $ 16;
      function='label ';
	  color='black';
	  style="'Courier'";
	  /* Set up treatment names at top of boxes */
	  if first.indexvar2 then do;
	    size=1.75;
        halfway= (((&xcntr - 1)/2) + 1) + ((indexvar2-1)* &xcntr );
        x=halfway;
		y=100;
		ysys='5';
		xsys='2';
		text=put(indexvar2,?? zvariable.);
		output;
      end;
	  /* Set up by-grouping header and title */
	  if _n_=1 then do;
	    x=50;
        xsys='3'; ysys='3'; 
	    y=100; text=put(indexvar1,?? byvariable.); size=1.5; position='8'; output;
  	    position='2'; text="&title"; size=2.5; output;
		style="'Courier'"; x=50; y=0; position='5'; text="&xaxislabel"; size=2; angle=0; output; 
		style="'simplex'"; x=1; y=50; position='5'; text="&yaxislabel"; size=2; angle=90; output; 
	  end;
	  /* Set up red circles for each data point */
	  position='5'; 
      function='symbol';
	  xsys='2';
	  ysys='2';
	  x=indexvar-.2;
	  y=&yvar;
	  color='red ';
	  %circle(indexvar-.2,&yvar,.3,red);
	  output;
    run;

	/* Create boxplot and call in annotation dataset */
    title ' ';
	proc sort data=pdlab out=pdlab;
	  by indexvar;
	run;

	/* If reference line specified */
	%if &hrefline^= %then %do;

      proc gplot data=pdlab gout=grph_cat;
        where indexvar1=&i;
	    format indexvar1 byvariable. indexvar xvariable. indexvar2 zvariable. &formats;
        plot &yvar*indexvar &yvar*indexvar/haxis=axis1 vaxis=axis2 href=&refbars anno=anno name="graph&i.a" vref=(&hrefline);
      run;
      quit;

	%end;
	/* If reference line not specified */
	%if &hrefline= %then %do;
      proc gplot data=pdlab gout=grph_cat;
        where indexvar1=&i;
	    format indexvar1 byvariable. indexvar xvariable. indexvar2 zvariable. &formats;
        plot &yvar*indexvar &yvar*indexvar/haxis=axis1 vaxis=axis2 href=&refbars anno=anno name="graph&i.a";
      run;
      quit;
	%end;
  %end;

  /* Replay out each by-grouping iteration graph also calling in the header information created earlier */
  ods pdf file="%unquote(&outputpathfilename).pdf";
    %local j;
    %do j=1 %to &bycnt;
	title ' ';
    proc greplay igout=grph_cat gout=grph_cat tc=grph_cat nofs ;
        template=newtemp ;
        treplay 1:graph&j.a 2:tbox ;
      run;
      quit;
    %end;
  ods pdf close;


    %local k;
    %do k=1 %to &bycnt;
	  proc greplay igout=grph_cat nofs;
        delete graph&k.a;
	  quit;
	%end;


%end;

/* If no by-variables specified */
%if &byvar= %then %do;

    data anno;
	  set pdlab;
	run;

	proc sort data=anno;
	  by indexvar2;
	run;

	/* Set up annotation */
	data anno;
	  set anno;
	  length style $ 16;
	  by indexvar2;
	  format indexvar xvariable. indexvar2 zvariable. &formats;
	  length text $ 256;
      function='label ';
	  color='black';
	  style="'Courier'";
	  /* Set up treatment names at top of boxes */
	  if first.indexvar2 then do;
	    size=1.75;
        halfway= (((&xcntr - 1)/2) + 1) + ((indexvar2-1)* &xcntr );
        x=halfway;
		y=100;
		ysys='5';
		xsys='2';
		text=put(indexvar2,?? zvariable.);
		output;
      end;
	  /* Set up title */
	  if _n_=1 then do;
	    x=50;
        xsys='3'; ysys='3'; 
	    y=100; size=1.5;
  	    position='2'; text="&title"; size=2.5; output;
		style="'Courier'"; x=50; y=0; position='5'; text="&xaxislabel"; size=2; angle=0; output; 
		style="'simplex'"; x=1; y=50; position='5'; text="&yaxislabel"; size=2; angle=90; output; 
	  end;
	  /* Set up red circles for each data point */
	  position='5'; 
      function='symbol';
	  xsys='2';
	  ysys='2';
	  x=indexvar-.2;
	  y=&yvar;
	  color='red ';
	  %circle(indexvar-.2,&yvar,.3,red);
	  output;
    run;

	/* Create boxplot and call in annotation dataset */
    title ' ';
	/* If reference line specified */
	%if &hrefline^= %then %do;
      proc gplot data=pdlab gout=grph_cat;
	    format indexvar xvariable. indexvar2 zvariable. &formats;
        plot &yvar*indexvar &yvar*indexvar/haxis=axis1 vaxis=axis2 href=&refbars anno=anno name="Graph1a" vref=(&hrefline);
      run;
      quit;
	%end;
	/* If reference line not specified */
	%if &hrefline= %then %do;
      proc gplot data=pdlab gout=grph_cat;
	    format indexvar xvariable. indexvar2 zvariable. &formats;
        plot &yvar*indexvar &yvar*indexvar/haxis=axis1 vaxis=axis2 href=&refbars anno=anno name="Graph1a";
      run;
      quit;
	%end;

  /* Replay out each by-grouping iteration graph also calling in the header information created earlier */
  ods pdf file="%unquote(&outputpathfilename).pdf";
	title ' ';
    proc greplay igout=grph_cat gout=grph_cat tc=grph_cat nofs ;
        template=newtemp ;
        treplay 1:Graph1a 2:tbox ;
      run;
      quit;
  ods pdf close;

%end;
%mend pd_boxplotindividual;

%pd_boxplotindividual(libname=U:\MetabolicBook\FinalData, 
              dsn=aucwm (where=(pdan='GLUCOSE')),  
			  byvar=,
			  xvar=perstady,   
			  yvar=auc24wm, 
			  zvar=atrtcd atrtgrp,
              formats=,
			  title=Box-Plot of Weighted Mean Glucose to Examine Response,
			  xaxislabel=Day,
			  yaxislabel=Glucose (MG/DL),
			  hrefline=,
			  outputpathfilename=U:\MetabolicBook\FinalPrograms\4BoxPlot\BoxPlotIndividual
              );
			  
